@@ -140,6 +140,23 @@
 #define INLINE inline
 #endif
 
+// '__has_attribute' macro was introduced by clang. later picked up by gcc.
+// If not supported by the current toolchain, define it to zero.
+#ifndef __has_attribute
+#define __has_attribute(x) 0
+#endif
+
+// Disables undefined behavior analysis for a function.
+// GCC 4.9+ uses __attribute__((no_sanitize_undefined))
+// clang uses __attribute__((no_sanitize("undefined")))
+#if defined(__GNUC__) && ((__GNUC__ * 100 + __GNUC_MINOR__) >= 409)
+#define UHDR_NO_SANITIZE_UNDEFINED __attribute__((no_sanitize_undefined))
+#elif __has_attribute(no_sanitize)
+#define UHDR_NO_SANITIZE_UNDEFINED __attribute__((no_sanitize("undefined")))
+#else
+#define UHDR_NO_SANITIZE_UNDEFINED
+#endif
+
 static const uhdr_error_info_t g_no_error = {UHDR_CODEC_OK, 0, ""};
 
 namespace ultrahdr {
@@ -174,7 +191,7 @@ typedef struct uhdr_raw_image_ext : uhdr_raw_image_t {
 /**\brief extended compressed image descriptor */
 typedef struct uhdr_compressed_image_ext : uhdr_compressed_image_t {
   uhdr_compressed_image_ext(uhdr_color_gamut_t cg, uhdr_color_transfer_t ct,
-                            uhdr_color_range_t range, unsigned sz);
+                            uhdr_color_range_t range, size_t sz);
 
  private:
   std::unique_ptr<ultrahdr::uhdr_memory_block> m_block;
@@ -324,6 +341,8 @@ bool isBufferDataContiguous(uhdr_raw_image_t* img);
 
 #endif
 
+uhdr_error_info_t uhdr_validate_gainmap_metadata_descriptor(uhdr_gainmap_metadata_t* metadata);
+
 }  // namespace ultrahdr
 
 // ===============================================================================================
@@ -356,6 +375,7 @@ struct uhdr_encoder_private : uhdr_codec_private {
   uhdr_enc_preset_t m_enc_preset;
   float m_min_content_boost;
   float m_max_content_boost;
+  float m_target_disp_max_brightness;
 
   // internal data
   std::unique_ptr<ultrahdr::uhdr_compressed_image_ext_t> m_compressed_output_buffer;
